// JHrestikiNuliki версія 1.2.
// Сергій Вовк. 2016.

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.List;
import java.util.ArrayList;

public class JHrestikiNuliki extends JFrame {
	private static final long serialVersionUID = 1L;
	private boolean isGameEnded;
	private JLabel statusLabel;
	private JButton newGameButton;
	private JButton[] gameButtons;

	public JHrestikiNuliki() {
		super("Хрестики нулики");

	    Font f = new Font("Serif", Font.PLAIN, 22);
	    statusLabel = new JLabel("Ваш хід.");
	    statusLabel.setFont(f);
	    statusLabel.setPreferredSize(new Dimension(200, 40));
	    isGameEnded = false;

	    newGameButton = new JButton("Нова гра");
	    newGameButton.setFont(f);
	    newGameButton.setVisible(false);
	    newGameButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e) {
            	for (JButton b : gameButtons) b.setText("");
                statusLabel.setText("Ваш хід.");
        	    newGameButton.setVisible(false);
                isGameEnded = false;
    			if (Math.round(Math.random()) == 0) simpleGo();
	    	}});

	    JPanel p = new JPanel(new FlowLayout());
	    p.add(statusLabel);
	    p.add(newGameButton);
	    ((FlowLayout)p.getLayout()).setHgap(20);
	    ((FlowLayout)p.getLayout()).setVgap(10);

	    JPanel buttonsPanel = new JPanel(new GridLayout(0, 3));

	    ActionListener a = new ActionListener() {
    		public void actionPerformed(ActionEvent e) {
    			gameButtonClick((JButton)e.getSource());
    		}};

 	    gameButtons = new JButton[9];
	    for (int i = 0; i < 9; i++) {
	    	gameButtons[i] = new JButton("");
	    	gameButtons[i].setName(Integer.toString(i));
	    	gameButtons[i].setFont(f);
	    	gameButtons[i].setPreferredSize(new Dimension(60, 60));
    		gameButtons[i].addActionListener(a);
	    	buttonsPanel.add(gameButtons[i]);
	    }

	    add(p, BorderLayout.WEST);
	    add(buttonsPanel, BorderLayout.SOUTH);
		setBounds(320, 200, 400, 300);
		setLocationRelativeTo(null);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
	}

	JButton[][] getButtonsArray() {
		return new JButton[][] {
			// По горизонталі:
			{gameButtons[0], gameButtons[1], gameButtons[2]},
			{gameButtons[3], gameButtons[4], gameButtons[5]},
			{gameButtons[6], gameButtons[7], gameButtons[8]},

			// По вертикалі:
			{gameButtons[0], gameButtons[3], gameButtons[6]},
			{gameButtons[1], gameButtons[4], gameButtons[7]},
			{gameButtons[2], gameButtons[5], gameButtons[8]},

			 // По диагоналі:
			{gameButtons[0], gameButtons[4], gameButtons[8]},
			{gameButtons[2], gameButtons[4], gameButtons[6]}};
	}

	// Функція setStatus визначає стан гри по стану кнопок.
	private void setStatus() {
		int i; JButton[] r; JButton a[][] = getButtonsArray();
        // Перевірка чи користувач або компьютер виграв.
        for (i = 0; i < 8; i++) {
			r = a[i];
			if ((!r[0].getText().isEmpty()) &&
					(r[1].getText() == r[0].getText()) &&
					(r[2].getText() == r[0].getText())) {
				statusLabel.setText(
					(r[0].getText() == "x") ? "Ви виграли!" : "Ви програли.");
				newGameButton.setVisible(true);
				isGameEnded = true;
                return;
            }
        }

		// Перевірка на нічийну.
		for (JButton b : gameButtons) if (b.getText().isEmpty()) return;
		statusLabel.setText("Нічийна.");
		newGameButton.setVisible(true);
		isGameEnded = true;
	}

	private void simpleGo() { // функція робить простий хід.
		if (gameButtons[4].getText().isEmpty() &&
				(Math.round(Math.random()) == 0)) {
			gameButtons[4].setText("0");
			setStatus();
			return;
		}

		List<JButton> emptyButtons = new ArrayList<JButton>();
		JButton[] cornerButtons = new JButton[] {
			gameButtons[0], gameButtons[2], gameButtons[6], gameButtons[8]};
		for (JButton b : cornerButtons)
			if (b.getText().isEmpty()) emptyButtons.add(b);

		if (emptyButtons.isEmpty()) {
			JButton[] a = new JButton[] {gameButtons[1], gameButtons[3],
			    gameButtons[4], gameButtons[5], gameButtons[7]};
			for (JButton b : a) if (b.getText().isEmpty()) emptyButtons.add(b);
		}

		if (emptyButtons.isEmpty()) return;
		if (emptyButtons.size() == 1) emptyButtons.get(0).setText("0");
		else emptyButtons.get((int)
			Math.round(Math.random() * (emptyButtons.size() - 1))).setText("0");
		setStatus();
	}

	private void gameButtonClick(JButton b) { // обробка натиснення кнопки.
		if (isGameEnded || !b.getText().isEmpty()) return;
		b.setText("x");
		setStatus();
		if (isGameEnded) return;
		// У масиві a визначено рядки кнопок для перевірки.
		int i; JButton[] r; JButton[][] a = getButtonsArray();

        // Перевірка щоб виграти.
		for (i = 0; i < 8; i++) {
			r = a[i];
            if (r[0].getText().isEmpty() && (r[1].getText() == "0") &&
            	(r[2].getText() == "0"))
				{ r[0].setText("0"); setStatus(); return; }
            if ((r[0].getText() == "0") && r[1].getText().isEmpty() &&
            	(r[2].getText() == "0"))
				{ r[1].setText("0"); setStatus(); return; }
            if ((r[0].getText() == "0") && (r[1].getText() == "0") &&
            	r[2].getText().isEmpty())
				{ r[2].setText("0"); setStatus(); return; }
        }

        // Перевірка щоб не дати виграти.
		for (i = 0; i < 8; i++) {
			r = a[i];
            if (r[0].getText().isEmpty() && (r[1].getText() == "x") &&
            	(r[2].getText() == "x"))
				{ r[0].setText("0"); setStatus(); return; }
            if ((r[0].getText() == "x") && r[1].getText().isEmpty() &&
            	(r[2].getText() == "x"))
				{ r[1].setText("0"); setStatus(); return; }
            if ((r[0].getText() == "x") && (r[1].getText() == "x") &&
            	r[2].getText().isEmpty())
				{ r[2].setText("0"); setStatus(); return; }
        }

		simpleGo();
	}

	public static void main(String[] args) {
		JHrestikiNuliki app = new JHrestikiNuliki();
		app.setVisible(true);
	}
}
